//! The RTFM framework
//!
//! What it covers:
//! - Priority based scheduling
//! - Message passing

#![no_main]
#![no_std]

extern crate panic_halt;

use cortex_m::{asm, iprintln};

extern crate stm32f4xx_hal as hal;
use crate::hal::prelude::*;
use crate::hal::serial::{config::Config, Event, Rx, Serial, Tx};
use hal::stm32::ITM;

use crate::hal::gpio::{gpioa::PA5, Output, PushPull};

use nb::block;
use rtfm::{app, Instant};

// Our error type
#[derive(Debug)]
pub enum Error {
    RingBufferOverflow,
    UsartSendOverflow,
    UsartReceiveOverflow,
}

enum CmdType {
    Off,
    On,
    Set,
}

pub struct Cmd {
    cmd_type: CmdType,
    half_period: u32,
}

impl Cmd {
    fn parse(&mut self, cmd_byte: [u8; 10], cmd_length: usize) {
        if cmd_byte.starts_with(&[b'o', b'n']) {
            self.cmd_type = CmdType::On;
            self.half_period = Cmd::freq_to_half_period(1);
        } else if cmd_byte.starts_with(&[b's', b'e', b't', b' ']) {
            let mut tmp = 0;
            for c in &cmd_byte[4..cmd_length] {
                if c.is_ascii_digit() {
                    tmp = 10 * tmp + (c - b'0') as u32;
                } else {
                    return;
                }
            }
            self.cmd_type = CmdType::Set;
            self.half_period = Cmd::freq_to_half_period(tmp);
        } else {
            self.cmd_type = CmdType::Off;
            self.half_period = Cmd::freq_to_half_period(1);
        }
    }

    fn freq_to_half_period(freq: u32) -> u32 {
        if freq == 0 {
            return Cmd::freq_to_half_period(1);
        } else {
            return SYSTEM_CLOCK_FREQ / (2 * freq);
        }
    }
}

const ASCII_CR: u8 = 13;
const ASCII_LF: u8 = 10;

const CMD_BUFFER_LENGTH: usize = 10;

const SYSTEM_CLOCK_FREQ: u32 = 16000000;

#[app(device = hal::stm32)]
const APP: () = {
    // Late resources
    static mut TX: Tx<hal::stm32::USART2> = ();
    static mut RX: Rx<hal::stm32::USART2> = ();
    static mut ITM: ITM = ();
    static mut IS_LED_ON: bool = ();
    static mut LED: PA5<Output<PushPull>> = ();
    static mut B: [u8; CMD_BUFFER_LENGTH] = ();
    static mut B_INDEX: usize = ();
    static mut CURRENT_CMD: Cmd = ();

    // init runs in an interrupt free section>
    #[init(spawn = [off])]
    fn init() {
        let stim = &mut core.ITM.stim[0];
        iprintln!(stim, "bare10");

        let rcc = device.RCC.constrain();

        // 16 MHz (default, all clocks)
        let clocks = rcc.cfgr.freeze();

        let gpioa = device.GPIOA.split();

        let tx = gpioa.pa2.into_alternate_af7();
        let rx = gpioa.pa3.into_alternate_af7(); // try comment out

        let mut led = gpioa.pa5.into_push_pull_output();
        led.set_low();

        let mut serial = Serial::usart2(
            device.USART2,
            (tx, rx),
            Config::default().baudrate(115_200.bps()),
            clocks,
        )
        .unwrap();

        // generate interrupt on Rxne
        serial.listen(Event::Rxne);
        // Separate out the sender and receiver of the serial port
        let (tx, rx) = serial.split();

        let _ = spawn.off();
        // Our split serial
        TX = tx;
        RX = rx;

        // LED
        LED = led;
        IS_LED_ON = false;
        CURRENT_CMD = Cmd {
            cmd_type: CmdType::Off,
            half_period: Cmd::freq_to_half_period(1),
        };
        // CMD Interpretor
        B = [0u8; 10];
        B_INDEX = 0;

        // For debugging
        ITM = core.ITM;
    }

    // idle may be interrupted by other interrupt/tasks in the system
    #[idle]
    fn idle() -> ! {
        loop {
            asm::wfi();
        }
    }

    #[task(priority = 1, capacity = 10, resources = [IS_LED_ON, B, B_INDEX, CURRENT_CMD, ITM], spawn = [on, off])]
    fn cmd_interpretor(byte: u8) {
        let is_led_on = *resources.IS_LED_ON;
        let mut b = *resources.B;
        let mut b_i = *resources.B_INDEX;
        let mut cmd = resources.CURRENT_CMD;

        if b_i >= CMD_BUFFER_LENGTH {
            b = [0u8; 10];
            b_i = 0;
        }

        match byte {
            ASCII_CR => {
                cmd.parse(b, b_i);
                b = [0u8; 10];
                b_i = 0;
            },
            ASCII_LF => {
                cmd.parse(b, b_i);
                b = [0u8; 10];
                b_i = 0;
            },
            _ => {
                b[b_i] = byte;
                b_i += 1;
            },
        };

        let stim = &mut resources.ITM.stim[0];
        iprintln!(stim, "byte: {:?}", byte);
        iprintln!(stim, "B: {:?}", b);
        iprintln!(stim, "B INDEX: {:?}", b_i);
        iprintln!(stim, "IS_LED_ON: {:?}", is_led_on);

        match cmd.cmd_type {
            CmdType::Off => iprintln!(stim, "CURRENT_CMD: off, {:?}", cmd.half_period),
            CmdType::On => iprintln!(stim, "CURRENT_CMD: on, {:?}", cmd.half_period),
            CmdType::Set => iprintln!(stim, "CURRENT_CMD: set, {:?}", cmd.half_period),
        }

        *resources.B = b;
        *resources.B_INDEX = b_i;
    }

    #[task(priority = 1, schedule = [off, on], resources = [CURRENT_CMD, LED, IS_LED_ON, ITM], spawn = [off])]
    fn on() {
        let cmd = resources.CURRENT_CMD;
        let stim = &mut resources.ITM.stim[0];
        iprintln!(stim, "entered on: {:?}", scheduled);


        match cmd.cmd_type {
            CmdType::On => {
                schedule.on(scheduled + (cmd.half_period).cycles());
                if *resources.IS_LED_ON == false {
                    resources.LED.set_high();
                    *resources.IS_LED_ON = true;
                }
            },
            CmdType::Off => {
                let _ = spawn.off();
            },
            CmdType::Set => {
                schedule.off(scheduled + (cmd.half_period).cycles());
                if *resources.IS_LED_ON == false {
                    resources.LED.set_high();
                    *resources.IS_LED_ON = true;
                }
            },
        }
    }

    #[task(priority = 1, schedule = [off, on], resources = [CURRENT_CMD, LED, IS_LED_ON, ITM], spawn = [on])]
    fn off() {
        let cmd = resources.CURRENT_CMD;
        let stim = &mut resources.ITM.stim[0];
        iprintln!(stim, "entered off: {:?}", scheduled);
        
        match cmd.cmd_type {
            CmdType::On => {
                let _ = spawn.on();
            },
            CmdType::Off => {
                schedule.off(scheduled + (cmd.half_period).cycles());
                if *resources.IS_LED_ON == true {
                    resources.LED.set_low();
                    *resources.IS_LED_ON = false;
                }
            },
            CmdType::Set => {
                schedule.on(scheduled + (cmd.half_period).cycles());
                if *resources.IS_LED_ON == true {
                    resources.LED.set_low();
                    *resources.IS_LED_ON = false;
                }
            },
        }
    }

    #[task(priority = 1, resources = [ITM])]
    fn trace_error(error: Error) {
        let stim = &mut resources.ITM.stim[0];
        iprintln!(stim, "{:?}", error);
    }

    #[interrupt(priority = 3, resources = [RX], spawn = [trace_error, cmd_interpretor])]
    fn USART2() {
        let rx = resources.RX;

        match rx.read() {
            Ok(byte) => {
                if spawn.cmd_interpretor(byte).is_err() {
                    let _ = spawn.trace_error(Error::RingBufferOverflow);
                }
            }
            Err(_err) => {
                let _ = spawn.trace_error(Error::UsartReceiveOverflow);
            }
        }
    }

    // Set of interrupt vectors, free to use for RTFM tasks
    // 1 per priority level suffices
    extern "C" {
        fn EXTI0();
        fn EXTI1();
    }
};

// 0. Compile and run the project at 16MHz in release mode
//    make sure its running (not paused).
//
//    > cargo build --example bare9 --features "hal rtfm" --release
//    (or use the vscode build task)
//
//    Connect a terminal program.
//    Verify that it works as bare9.
// 
// 1. Now, comment out the loop in `trace_data`.
//    The loop is just there to simulate some workload...
//  
//    Try now to send a sequence `abcd`
//
//    Did you loose any data (was the data correctly echoed)?
//
//    ** No, the data was correctly echoed. **
//
//    Was the data correctly traced over the ITM?
//
//    ** No, only ab was traced. c and d generated RingBufferOverflow. **
//
//    Why did you loose trace information?
//
//    ** The interrupt has higher priority so it takes over from the running
//    trace_data task, but that task has not finished its nop-loop started
//    in a previous interrupt. Therefore the block!() evaluates to an error. **
//
//    Commit your answers (bare10_1)
//
// 2. Read the RTFM manual (book).
//    Figure out a way to accomdate for 4 outstanding messages to the `trace_data` task.
//
//    Verify that you can now correctly trace sequences of 4 characters sent.
//
//    Can you think of how to determine a safe bound on the message buffer size?
//    (Safe meaning, that message would never be lost due to the buffer being full.)
//
//    What information would you need?
//
//    ** The time each byte needs to be processed (call it A) and the time it takes for
//    a byte to arrive after the previous arrived (call it B). We need space for approximately
//    A/B tasks to run simultaneously. **
//    
//    Commit your answers (bare10_2)
//
// 3. Implement a command line interpreter as a new task.
//    It should:  
//    - have priority 1.
//    - take a byte as an argument (passed from the USART2 interrupt).
//    - have a local buffer B of 10 characters
//    - have sufficent capacity to recieve 10 charactes sent in a sequence
//    - analyse the input buffer B checking the commands
//      set <int> <RETURN>       // to set blinking frequency
//      on <RETURN>              // to enable the led blinking
//      of <RETURN>              // to disable the led blinking
// 
//      <int> should be decoded to an integer value T, and <RETURN> accept either <CR> or <LF>.
//
//    The set value should blink the LED in according the set value in Herz, 
//    (so `set 1 <RETURN>` should blink with 1Hz)
//    
//    Tips:
//    Create two tasks, (`on', that turns the led on, and a task `off` that turns the led off).
//    `on` calls `off` with a timer offset (check the RTFM manual).
//    `off` calls `on` with a timer offset.
//
//    The timing offset can implemented as a shared resource T between the command line interpreter and
//    the  'on/off ' tasks. From `init` you can give an initial timing offset T, and send an
//    initial message to `on` triggering the periodic behavior. 
//
//    The 'on/off ' tasks can have a high priority 4, and use locking (in the low prio task)
//    parsing the input. This way, the led will have a very low jitter.
//
//    (You can even use an atomic data structure, which allows for lock free access.)
//    
//  
//    The on/off is easiest impemented by having another shared variable used as a condition
//    for the `on` task to set the GPIO. Other solutions could be to stop the sequence (i.e.)
//    contiditionally call the `off` task instead. Then the command interpreted would
//    trigger a new sequence when an "on" command is detected. (Should you allow multiple, overlapping)
//    sequences? Why not, could be cool ;)
//    The main advantage of the stopping approach is that the system will be truly idle
//    if not blinking, and thus more power efficient.
//
//    You can reuse the code for setting up and controlling the GPIO/led.
//
//    You can come up with various extensions to this application, setting the
//    the duty cycle (on/off ratio in %), etc.
//
//    Commit your solution (bare10_3)