//! bare7.rs
//! 
//! Serial echo
//!
//! What it covers:
//! - changing the clock using Rust code
//! - working with the svd2rust API
//! - working with the HAL (Hardware Abstraction Layer)
//! - USART polling (blocking wait)

#![deny(unsafe_code)]
#![no_main]
#![no_std]

extern crate panic_halt;

use cortex_m::iprintln;
use cortex_m_rt::entry;
use nb::block;

extern crate stm32f4xx_hal as hal;
use crate::hal::prelude::*;
use crate::hal::serial::{config::Config, Serial};

#[entry]
fn main() -> ! {
    let mut c = hal::stm32::CorePeripherals::take().unwrap();
    let stim = &mut c.ITM.stim[0];
    iprintln!(stim, "bare7");

    let p = hal::stm32::Peripherals::take().unwrap();


    let rcc = p.RCC;

    // Output MCO2 to PC9 as SYSCLK/4:
    rcc.cfgr.modify(|_, w| w.mco2().sysclk().mco2pre().div1());

    // Enable GPIOC:
    rcc.ahb1enr.modify(|_, w| w.gpiocen().enabled());
    let gpioc = p.GPIOC;

    // Set Port C pin 9 to alternate function MCO2.
    gpioc.moder.modify(|_, w| w.moder9().alternate());
    gpioc.ospeedr.modify(|_, w| w.ospeedr9().low_speed());

    // Now lock the clocks:
    let rcc = rcc.constrain();
    let clocks = rcc.cfgr.sysclk(84.mhz()).pclk1(42.mhz()).pclk2(42.mhz()).freeze();

    let gpioa = p.GPIOA.split();

    let tx = gpioa.pa2.into_alternate_af7();
    let rx = gpioa.pa3.into_alternate_af7(); // try comment out
    // let rx = gpioa.pa3.into_alternate_af6(); // try uncomment

    let serial = Serial::usart2(
        p.USART2,
        (tx, rx),
        Config::default().baudrate(115_200.bps()),
        clocks,
    )
    .unwrap();

    // Separate out the sender and receiver of the serial port
    let (mut tx, mut rx) = serial.split();

    loop {
        match block!(rx.read()) {
            Ok(byte) => {
                iprintln!(stim, "Ok {:?}", byte);
                tx.write(byte).unwrap(); 
            }
            Err(err) => {
                iprintln!(stim, "Error {:?}", err);
            }
        }
    }
}

// 0. Background reading:
//    STM32F401xD STM32F401xE, section 3.11
//    We have two AMBA High-performance Bus (AHB)
//    APB1 low speed bus (max freq 42 MHz)
//    APB2 high speed bus (max frex 84 MHz)
//
//    RM0368 Section 6.2
//    Some important/useful clock acronymes and their use:
//
//    SYSCLK - the clock that drives the `core`
//    HCLK   - the clock that drives the AMBA bus(es), memory, DMA, trace unit, etc.
//
//    Typically we set HCLK = SYSCLK / 1 (no prescale) for our applications
//
//    FCLK   - Free running clock runing at HCLK
//
//    CST    - CoreSystemTimer drives the SysTick counter, HCLK/(1 or 8)
//    PCLK1  - The clock driving the APB1 (<= 42 MHz)
//             Timers on the APB1 bus will be triggered at PCLK1 * 2
//    PCLK2  - The clock driving the APB2 (<= 84 MHz)
//             Timers on the APB2 bus will be triggered at PCLK2
//
//    Compliation:
//    > cargo build --example bare7 --features "hal"
//    (or use the vscode build task)
//
//    The "hal" feature enables the optional dependency stm32f4xx-hal".
//
//    Cargo.toml:
// 
//    [dependencies.stm32f4]
//    version = "0.5.0"
//    features = ["stm32f413", "rt"]
//    optional = true
// 
//    [dependencies.stm32f4xx-hal]
//    git = "https://github.com/stm32-rs/stm32f4xx-hal.git"
//    version = "0.2.8"
//    features = ["stm32f413", "rt"]
//    optional = true
//
//    [features]
//    pac = ["stm32f4"]
//    hal = ["stm32f4xx-hal"]
//
//    Notice, stm32f4xx-hal internally enables the dependency to stm32f4, 
//    so we don't need to explicitly enable it.
//
//    The HAL provides a generic abstraction over the whole stm32f4 family,
//    as previously we use the stm32f413, since it also covers stm32f401/stm32f411.
//
// 1. The rcc.cfgr.x.freeze() sets the clock according to the configuration x given.
//
//    rcc.cfgr.freeze(); sets a default configuration.
//    sysclk = hclk = pclk1 = pclk2 = 16MHz
//
//    What is wrong with the following configurations?
//
//    rcc.cfgr.sysclk(64.mhz()).pclk1(64.mhz()).pclk2(64.mhz()).freeze();
//
//    ** PCLK1 must be lower than 42 MHz. **
//
//    rcc.cfgr.sysclk(84.mhz()).pclk1(42.mhz()).pclk2(64.mhz()).freeze();
//
//    ** With sysclk at 84 MHz, it is impossible to get PCLK2 at 64 MHz with only a
//    prescaler that divides by integer values. Closest PCLK2 are 84 MHz and 42 MHz. **
//
//    Commit your answers (bare7_1)
//
//    Tip: You may use `stm32cubemx` to get a graphical view for experimentation.
//    This is a good tool when designing your PCB (as pinouts are given).
//
// 2. Now give the system with a valid clock, sysclk of 84 MHz.
//
//    Include the code for outputting the clock to MCO2.
//
//    Repeat the expermient bare6_2.
//
//    What is the frequency of MCO2 read by the oscilloscope.
//
//    ** 21.053 MHz **
//
//    Compute the value of SYSCLK based on the oscilloscope reading.
//
//    ** MCO2 * MCO2PRE = 21.053 * 4 which is about 84 MHz. **
//
//    What is the peak to peak reading of the signal.
//
//    ** 6.20 including the ringing, and without ringing 3.1 V **
//
//    Make a screen dump or photo of the oscilloscope output.
//    Save the the picture as "bare_6_84mhz_high_speed"
//
//    Commit your answers (bare7_2)
//
// 3. Now reprogram the PC9 to be "Low Speed", and re-run at 84Mz.
//
//    Did the frequency change in comparison to assignment 5? << Assuming last task
//
//    ** The frequency is the same. BUT, changing the divider from 4 to 1, meaning
//    that MCO2 = SYSCLK = 84 MHz, the output is zero, while it is not zero with
//    "very_high_speed()" on. **
//
//    What is the peak to peak reading of the signal (and why did it change)?
//
//    ** Zero at MCO2 = 84 MHz and low_speed mode. 3.95 V at MCO = 21 MHz. 
//    The voltage changed because more current is required to drive the CLK up and
//    down at higher frequency. The speed setting exists to be able to conserve
//    power when only little power is needed, i.e. at low frequencies. The current
//    in the low speed setting can't drive the CLK all the way up to nominal voltage,
//    charge, that fast. **
//
//    Make a screen dump or photo of the oscilloscope output.
//    Save the the picture as "bare_6_84mhz_low_speed".
//
//    ** It is the MCO2 = 84 MHz, low_speed mode I put in the pictures/ folder. **
//
//    Commit your answers (bare7_3)
//
// 4. Revisit the `README.md` regarding serial communication.
//    start a terminal program, e.g., `moserial`.
//    Connect to the port
//
//    Device       /dev/ttyACM0
//    Baude Rate   115200
//    Data Bits    8
//    Stop Bits    1
//    Parity       None
//    
//    This setting is typically abbreviated as 115200 8N1.
//
//    Run the example, make sure your ITM is set to 84MHz.
//
//    Send a single character (byte), (set the option No end in moserial).
//    Verify that sent bytes are echoed back, and that ITM tracing is working.
// 
//    If not go back check your ITM setting, clocks etc.
//
//    Try sending: "abcd" as a single sequence, don't send the quation marks, just abcd.
//
//    What did you receive, and what was the output of the ITM trace.
//
//    ** Serial: abcd
//       ITM: Ok 97 Ok 98 Ok 99 Ok 100 **
//
//    Explain why the buffer overflows.
//
//    ** There was no buffer overflow. I instead wrote 496 bytes, it failed after 336 ITM
//    outputs. It failed to echo after 425 characters. It fails because the readout (and write)
//    of the registers take time, and if bytes are added faster than the cpu reads them, the
//    register can be cleared to make way for new incoming bytes even before all bytes
//    have been read by the program. **
//
//    commit your answers (bare7_4)
//
//    Discussion:
//    Common to all MCUs is that they have multiple clocking options.
//    Understanding the possibilities and limitations of clocking is fundamental
//    to designing both the embedded hardware and software. Tools like
//    `stm32cubemx` can be helpful to give you the big picture.
//    You will likely finding it useful when designing your board (e.g., checking
//    what functionality can be associated to each pin etc.)
//
//    The `stm32f4xx-hal` gives you an abstraction for programming,
//    setting up clocks, assigning pins, etc.
//
//    The hal is under development, and already covers some basic functionality
//    like serial communication. Still, in order to fully understand what is
//    going on under the hood you need to check the documentation (data sheets,
//    user manuals etc.)
//
//    Your crate can be autmatically documentedthread:
//
//    $ cargo doc --open --features "hal"
//
//    This will document both your crate and its dependencies besides the `core` library.
//
//    You can open the `core` library documentation by
//
//    $ rustup doc
//
//    or just show the path to the doc (to open it manually)
//
//    $ rustup doc --path
